<?php
/**
 * @author     Thomas Meyer
 * @licence    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_sdk extends plugin 
{

    /**
     * Default event handler
     *
     * @param string $event name of the called event in lower-case
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function _default($event, $ARGS) 
    {
        return true;
    }

    /**
     * Type: read
     * Permission: 100
     * Templates:  sdk_read_form
     *
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function sdk_read_form ($ARGS) 
    {
        global $YANA;
        $dir = $YANA->plugins->get($this->name, 'LOGOS');
        $dir->read();
        $YANA->setVar('FILES',   $dir->get());
        $YANA->setVar('IMG_SRC', $dir->getFilename());
        $config = $YANA->plugins->get($this->name, 'CONFIGFILE');
        $config->read();
        $YANA->setVar('PLUGIN',  $config->get());
    }

    /**
     * Type: write
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function sdk_write_plugin ($ARGS) 
    {
        global $YANA;
        $config = $YANA->plugins->get($this->name, 'CONFIGFILE');
        $config->read();
        if (!$config->exists()) {
            $config->create();
        }
        unset($ARGS[strtolower(session_name())]);
        unset($ARGS['action']);
        unset($ARGS['id']);
        unset($ARGS['table']);
        $config->insert($ARGS);
        $config->write();
        $dir = $YANA->plugins->get($this->name, 'LOGOS');

        if (!preg_match('/^[\d\w-_]+$/i', $ARGS['plugin_id'])) {
            die('<h1>Fehler</h1><p>Ungltige Zeichen in ID: &quot;'.$ARGS['plugin_id'].'&quot;</p><p>Nur alphanumerische Zeichen, sowie &quot;-&quot; &quot;_&quot; erlaubt</p>');
        }
        if (!preg_match('/^[\d\w-_ \(\)]+$/i', $ARGS['name'])) {
            die('<h1>Fehler</h1><p>Ungltige Zeichen in Namen der Anwendung: &quot;'.$ARGS['name'].'&quot;</p><p>Nur alphanumerische Zeichen, sowie &quot;&szlig;&quot; &quot;&auml;&quot; &quot;&ouml;&quot; &quot;&uuml;&quot; &quot; &quot; &quot;-&quot; &quot;_&quot; erlaubt</p>');
        }
        if (!preg_match('/^[\d\w-_\.]+\.(gif|jpg|png)$/i', $ARGS['grafik'])) {
            die('<h1>Fehler</h1><p>Die ausgew&auml;hlte Grafik: &quot;'.$ARGS['grafik'].'&quot; ist ung&uuml;ltig, weil der Dateiname nicht erlaubte Zeichen enth&auml;lt oder die Datei eine falsches Format hat.</p><p>Es sind nur Dateinamen, keine Pfadangaben oder Verzeichniswechsel erlaubt. Erlaubte Dateiendungen sind gif, jpg und png.</p>');
        }
        if (!is_numeric($ARGS['version_nr'])) {
            die('<h1>Fehler</h1><p>Die Versionsnummer: &quot;'.$ARGS['version_nr'].'&quot; ist ung&uuml;ltig.</p><p>Die Versionsnummer muss eine Zahl sein.</p>');
        }
        if (!preg_match('/^[\d\w-_]*$/i', $ARGS['parent'])) {
            die('<h1>Fehler</h1><p>Die ID des ben&ouml;tigten Plugin: &quot;'.$ARGS['parent'].'&quot; ist ung&uuml;ltig.</p><p>Nur alphanumerische Zeichen, sowie &quot;-&quot; &quot;_&quot; erlaubt</p>');
        }
        if (!preg_match('/^[\d\w-_]*$/i', $ARGS['package'])) {
            die('<h1>Fehler</h1><p>Die ID des Anwendungspakets: &quot;'.$ARGS['package'].'&quot; ist ung&uuml;ltig.</p><p>Nur alphanumerische Zeichen, sowie &quot;-&quot; &quot;_&quot; erlaubt</p>');
        }
        if (!preg_match('/^(primary|default|config|read|write|security)$/i', $ARGS['type'])) {
            die('<h1>Fehler</h1><p>Der gew&auml;hlte Anwendungstyp: &quot;'.$ARGS['type'].'&quot; ist unbekannt.</p><p>Empfehlung: verwenden Sie im Zweifelsfall den Typ &quot;default&quot;</p>');
        }
        if (!is_numeric($ARGS['priority']) || $ARGS['priority'] < -1 || $ARGS['priority'] > 3) {
            die('<h1>Fehler</h1><p>Die Priorit&auml;t der Ausf&uuml;hrung: &quot;'.$ARGS['priority'].'&quot; hat einen ung&uuml;ltigen Wert.</p><p>Dies muss eine Zahl zwischen -1 und 3 sein.</p>');
        }

        $plgDir  = new dirStream($YANA->getVar('PLUGINDIR').$ARGS['plugin_id'].'/');
        $sknDir  = new dirStream($YANA->getVar('SKINDIR').'default/'.$ARGS['plugin_id'].'/');
        $sknFile = new smlfile($YANA->getVar('SKINDIR').'default/'.$ARGS['plugin_id'].'.config');
        $plgFile = new smlfile($YANA->getVar('PLUGINDIR').$ARGS['plugin_id'].'.config');

        if (!empty($ARGS['overwrite'])) {
            $plgDir->delete(true);
            $sknDir->delete(true);
            $sknFile->delete();
            $plgFile->delete();
        } elseif ($plgFile->exists() || $plgDir->exists()) {
            die('<h1>Fehler</h1><p>Ein Plugin mit der ID: &quot;'.$ARGS['plugin_id'].'&quot; existiert bereits.</p><p>Falls Sie ein Plugin ersetzen m&ouml;chten, l&ouml;schen Sie zuerst bereits vorhandene alte Dateien.</p>');
        }
        unset($ARGS['overwrite']);

        $drvFile   = new smlfile(         $plgDir->getFilename().'_drive.config');
        $phpFile   = new secureFileStream($plgDir->getFilename().'plugin.php');

        $INFO             = array();
        $DRIVE            = array();
        $CUSTOM_INTERFACE = array();
        $AUTO_INTERFACE   = array();
        $INTERFACE        = array();
        $COPY             = array();
        $DATABASE         = array();
        $SKIN             = array();

        { /* ID */
    
            $INFO['ID'] = $ARGS['plugin_id'];
            unset($ARGS['plugin_id']);
    
        }
    
        { /* NAME */
    
            $INFO['NAME'] = $ARGS['name'];
            unset($ARGS['name']);
    
        }
    
        { /* GRAFIK */
    
            $endung = '.'.preg_replace('/^.*\./', '', $ARGS['grafik']);
            $i = count($COPY);
            $COPY[$i]['src']  = $dir->getFilename().$ARGS['grafik'];
            $COPY[$i]['dest'] = $plgDir->getFilename().'preview'.$endung;
            $INFO['GRAFIK'] = '%PLUGINDIR%/'.$INFO['ID'].'/'.'preview'.$endung;
            unset($endung);
            unset($i);

        }
    
        { /* VERSION */
    
            $INFO['VERSION'] = strip_tags(nl2br($ARGS['version']));
            unset($ARGS['version']);
    
        }
    
        { /* VERSION_NR */
    
            $INFO['VERSION_NR'] = strip_tags(nl2br($ARGS['version_nr']));
            unset($ARGS['version_nr']);
    
        }
    
        { /* AUTOR */
    
            if (!empty($ARGS['autor'])) $INFO['AUTOR'] = nl2br(strip_tags($ARGS['autor']));
            unset($ARGS['autor']);
    
        }
    
        { /* KONTAKT */

            if (!empty($ARGS['kontakt'])) $INFO['KONTAKT'] = nl2br(strip_tags($ARGS['kontakt']));
            unset($ARGS['kontakt']);
    
        }
    
        { /* BESCHREIBUNG */
    
            if (!empty($ARGS['beschreibung'])) $INFO['BESCHREIBUNG'] = str_replace("\n",'[br]',strip_tags($ARGS['beschreibung']));
            unset($ARGS['beschreibung']);
    
        }
    
        { /* UPDATE */
    
            if (!empty($ARGS['update'])) $INFO['UPDATE'] = nl2br(strip_tags($ARGS['update']));
            unset($ARGS['update']);
    
        }
    
        { /* PARENT */
    
            if (!empty($ARGS['parent'])) $INFO['PARENT'] = nl2br(strip_tags($ARGS['parent']));
            unset($ARGS['parent']);
    
        }
    
        { /* PACKAGE */
    
            if (!empty($ARGS['package'])) $INFO['PACKAGE'] = nl2br(strip_tags($ARGS['package']));
            unset($ARGS['package']);

        }
    
        { /* TYPE */
    
            $INFO['TYPE'] = nl2br(strip_tags($ARGS['type']));
            unset($ARGS['type']);
    
        }
    
        { /* PRIORITY */
    
            $INFO['PRIORITY'] = $ARGS['priority'];
            unset($ARGS['priority']);
    
        }

        /* CREATE FILES */
        $plgFile->create();
        $plgDir->create(777);
        $drvFile->create();
        $phpFile->create();
        $sknDir->create(663);

        { /* sql_file */
    
            if (!empty($_FILES['sqlfile']['tmp_name'])) {
                move_uploaded_file($_FILES['sqlfile']['tmp_name'],$YANA->getVar('DBDIR').'.install/mysql/'.$INFO['ID'].'.sql');
                $DRIVE['SQL'] = array();
                $DRIVE['SQL']['TYPE']      = 'file';
                $DRIVE['SQL']['REQUIRED']  = 'readable';
                $DRIVE['SQL']['AUTOMOUNT'] = 'no';
                $DRIVE['SQL']['SRC']       = 'database.sql';
                unset($ARGS['sqlfile']);
            } else {
            }

        }
    
        { /* CONFIG_TEMPLATE */
    
            if ($ARGS['config_template'] == 'true'/*  || $create_installation_routine */) {
    
                $optFile     = $YANA->plugins->get($this->name, 'CONFIGTEMPLATE');
                $optTemplate = new smartTemplate($optFile->getFilename());
                $optTemplate->insert('*',$INFO);
                $optTemplate->insert('CREATE_SETUP_ROUTINE',$ARGS['config_template'] == 'true');
                $html =  $optTemplate->get();
                
                $optDir  = new dirStream($plgDir->getFilename().'templates/');
                $optDir->create(330);
                $optFile = new secureFileStream($optDir->getFilename().'setup.html');
                $optFile->create();
                $optFile->insert($html);
                $optFile->write();
                $INFO['CONFIG_TEMPLATE'] = preg_replace("/^plugins\//", "", $optFile->getFilename());
                $i = count($COPY);
                $iconDir = $YANA->plugins->get($this->name, 'ICONS');
                $COPY[$i]['src']  = $iconDir->getFilename().preg_replace('/(.*)\..*/', "\\1", $ARGS['grafik']).'.gif';
                $COPY[$i]['dest'] = $optDir->getFilename().'setup.gif';

            }
            unset($ARGS['config_template']);
            unset($ARGS['grafik']);
    
        }

        if (!empty($_FILES['sourcefile']['tmp_name'])) { /* source_file */
    
            move_uploaded_file($_FILES['sourcefile']['tmp_name'],$YANA->getVar('DBDIR').$INFO['ID'].'.config');
            unset($ARGS['sourcefile']);
    
            $dbsFile = new smlfile($YANA->getVar('DBDIR').$INFO['ID'].'.config');
            $dbsFile->init();
            $DATABASE = $dbsFile->get('*');

        } else {      
    
            $DATABASE = array();
    
        }  
    
        { /* INTERFACE */
    
            if ($ARGS['interface']) {
                $raw_data = explode("\n", $ARGS['interface']);
    
                for ($i=0; $i<count($raw_data); $i++)
                {
                    $raw_data[$i] = explode(",",$raw_data[$i]);
                    $j = $raw_data[$i][0];
                    if (!empty($j)) {
                        if (isset($CUSTOM_INTERFACE[$j])) echo '<h1>Warnung</h1><p>Eine Aktion mit der Bezeichnung &quot;'.$j.'&quot; existiert mehrmals. Erste Version wurde verworfen.</p>';
                        $CUSTOM_INTERFACE[$j] = array();
                        $CUSTOM_INTERFACE[$j]['TYPE'] = $raw_data[$i][1];
                        $CUSTOM_INTERFACE[$j]['MODE'] = $raw_data[$i][2];
                        if(!empty($raw_data[$i][3])) {
                            $CUSTOM_INTERFACE[$j]['TEMPLATE'] = $raw_data[$i][3];
                        }
                        if(!empty($raw_data[$i][4])) {
                            $CUSTOM_INTERFACE[$j]['INSERT'] = $raw_data[$i][4];
                        }
                        $CUSTOM_INTERFACE[$j]['PERMISSION'] = (int) $raw_data[$i][5];
                    }
                }
            }
            unset($ARGS['interface']);
            unset($ARGS['interface_mode']);
        }

        { /* create virtual drive */
        
            $DRIVE['MAIN'] = array();
            $DRIVE['MAIN']['TYPE']      = 'executeable';
            $DRIVE['MAIN']['REQUIRED']  = 'readable';
            $DRIVE['MAIN']['AUTOMOUNT'] = 'yes';
            $DRIVE['MAIN']['SRC']       = 'plugin.php';
    
        }

        { /* create GUI and corresponding Interface entries for each table */

            $INPUT = mergeArrays($INFO,array('INTERFACE' => $CUSTOM_INTERFACE));
            $HTML_MENU = array();
            $HTML_MENU['table_list'] = array();
            $HTML_MENU['event_list'] = array();

            foreach ($DATABASE['TABLES'] as $KEY => $CURRENT) 
            {
                $KEY = strtolower($KEY);
                array_push($HTML_MENU['table_list'], $KEY);
                $DB_SRC = array('DATABASE' => $CURRENT['CONTENT'], 'TABLE' => $KEY, 'PRIMARY_KEY' => $CURRENT['PRIMARY_KEY']);

                { /* Edit */
                    $current_action = $this->_create('gui_edit.html', $INFO['ID'], 'read', 'edit', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    $this->_create('', $INFO['ID'], 'write', 'edit', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 75, 'table_name' => $KEY, 'name' => 'edit'));
                }
    
                { /* Delete */
                    $this->_create('', $INFO['ID'], 'write', 'delete', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                }
    
                { /* New */
                    $current_action = $this->_create('gui_new.html', $INFO['ID'], 'default', 'new', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    $this->_create('', $INFO['ID'], 'write', 'new', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'new'));
                }
    
                { /* Search */
                    $current_action = $this->_create('gui_search.html', $INFO['ID'], 'read', 'search', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'search'));
                }

                { /* View */
                    $current_action = $this->_create('gui_view.html', $INFO['ID'], 'read', 'read', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'list'));
                }
    
                { /* View */
                    $current_action = $this->_create('gui_view_seperated.html', $INFO['ID'], 'read', 'read_seperated', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'view'));
                }

            }
    
        }
        
        { /* create index file */

            $tplDir   = $YANA->plugins->get($this->name, 'TEMPLATES');
            $tplDir   = $tplDir->getFilename();
            $idxTemplate = new smartTemplate($tplDir.'gui_index.html');
            $idxTemplate->insert('*', $HTML_MENU);
            $idxTemplate->insert('plugin_id', $INFO['ID']);
            $IDX         =  $idxTemplate->get();

        }
    
        { /* create class skeleton */

            $classFile   = $YANA->plugins->get($this->name, 'CLASS');
            $phpTemplate = new smartTemplate($classFile->getFilename());
            $phpTemplate->insert('*', $INFO);
            $phpTemplate->insert('INTERFACE', $CUSTOM_INTERFACE);
            $phpTemplate->insert('AUTO_INTERFACE', $AUTO_INTERFACE);
            $phpTemplate->insert('DATABASE',$DATABASE);      
            $PHP         =  $phpTemplate->get();
    
        }
    
        { /* copy files */
    
            foreach ($COPY as $k => $v) 
            {
                copy($v['src'], $v['dest']) || die ('<h1>Fehler</h1><p>Die Datei '.$v['src'].' konnte nicht kopiert werden.</p>');
            }
            
        }
        
        { /* write files */
    
            $INTERFACE = mergeArrays($CUSTOM_INTERFACE, $INTERFACE);
            $plgFile->set(array('INFO' => $INFO, 'INTERFACE' => $INTERFACE));
            $plgFile->write() || die('<h1>Fehler</h1><p>Die Datei '.$plgFile->getFilename().' konnte nicht gespeichert werden.</p>');
    
            $drvFile->set($DRIVE);
            $drvFile->write() || die('<h1>Fehler</h1><p>Die Datei '.$drvFile->getFilename().' konnte nicht gespeichert werden.</p>');
    
            $phpFile->insert("<?php\n".$PHP."\n?>");
            $phpFile->write() || die('<h1>Fehler</h1><p>Die Datei '.$phpFile->getFilename().' konnte nicht gespeichert werden.</p>');
    
        }

        { /* create Skin file */
    
            $sknFile->create();
            $sknFile->set($SKIN);
            $sknFile->write() || die('<h1>Fehler</h1><p>Die Datei '.$sknFile->getFilename().' konnte nicht gespeichert werden.</p>');
    
    
        }

        { /* create Index file */

            $idxFile = new secureFileStream($sknDir->getFilename().'index.html');
            $idxFile->create();
            $idxFile->insert($IDX);
            $idxFile->write() || die('<h1>Fehler</h1><p>Die Datei '.$idxFile->getFilename().' konnte nicht gespeichert werden.</p>');

        }

        if (count($ARGS)>0) {

            $t = "<h1>Warnung</h1><p>Folgende unbekannte Eingaben wurden ignoriert. Datenverluste m&ouml;glich. Pr&uuml;fen Sie Ihr Formular.</p>";
    
            foreach ($ARGS as $k => $v) 
            {
                $t .= "$k =&gt; $v<br>\n";
            }
            
            $YANA->report(new warning($t));
            return false;
        
        } else {
         
            $YANA->report(new message("<h1>Erfolgreich</h1><p>Die Anwendung wurde erfolgreich generiert.</p>"));
            return true;
    
        }

    }


    /**
     * Type: write
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function sdk_write_config ($ARGS) 
    {
        global $YANA;

        $db = new bufferedDbStream();
        if ($db->exportStructure($ARGS["table"])) {
            die("<h1>Erfolgreich</h1><p>Die Datei ".$ARGS["table"]." wurde erfolgreich erzeugt.</p>");
        } else {
            die("<h1>Fehler</h1><p>Die Datei ".$ARGS["table"]." konnte nicht erzeugt werden.</p>");
        }
    }

    /**
     * @access private
     */
    function _create ($src, $id, $type, $name, &$INPUT, &$DATABASE, $mode = 0, &$sknDir, &$AUTO_INTERFACE, &$SKIN, &$INTERFACE, &$INFO) 
    {
        global $YANA;
        $tplDir   = $YANA->plugins->get($this->name, 'TEMPLATES');
        $tplDir   = $tplDir->getFilename();
        $action   = $this->_createActionName($id,$type,$name,$DATABASE['TABLE']);
        $skinName = strtoupper($id.'_'.$name."_".$DATABASE['TABLE']);
        $dest     = strtolower($action.'.html');

        if (!empty($src)) {

            { /* create GUI File */    
                $guiTemplate = new smartTemplate("$tplDir$src");
                $guiTemplate->insert('*',$INPUT);
                $guiTemplate->insert('*',$DATABASE);
                $guiTemplate->insert('ACTION',$action);
                $guiTemplate->insert('ON_NEW',$id."_write_new_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_EDIT',$id."_write_edit_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_DELETE',$id."_write_delete_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_SEARCH',$id."_read_search_".$DATABASE['TABLE']);
                $html =  $guiTemplate->get();
                $file = new secureFileStream($sknDir->getFilename().$dest);
                if (!$file->exists()) {
                    $file->create();
                    $file->insert($html);
                    $file->write() || die('<h1>Fehler</h1><p>Die Datei '.$file->getFilename().' konnte nicht gespeichert werden.</p>');
                }
            }

            { /* create Interface Entry */
                $INTERFACE[$action]['TYPE']       = $type;
                $INTERFACE[$action]['MODE']       = 0;
                $INTERFACE[$action]['INSERT']     = $skinName;
                $INTERFACE[$action]['PERMISSION'] = ($mode > 0) ? $mode : 0;
                $INTERFACE[$action]['ONSUCCESS'] = array();
                $INTERFACE[$action]['ONERROR'] = array();
                $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $action;
                $INTERFACE[$action]['ONERROR']['GOTO'] = $action;
            }

            { /* create Skin Entry */
                $SKIN[$skinName] = array('FILE' => $id.'/'.$dest);
            }

            { /* create Root entry */
                if (($type == "default" || $type == "read") && $mode == 0) {
                    $INFO['START']  = $action;
                }
            }

        } else {
            
            { /* create Interface Entry */
                $INTERFACE[$action]['TYPE']       = $type;
                $INTERFACE[$action]['MODE']       = 0;
                $INTERFACE[$action]['TEMPLATE']   = 'MESSAGE';
                $INTERFACE[$action]['PERMISSION'] = ($mode > 0) ? $mode : 0;
                $INTERFACE[$action]['ONSUCCESS'] = array();
                $INTERFACE[$action]['ONERROR'] = array();
                $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $action;
                $INTERFACE[$action]['ONERROR']['GOTO'] = $action;
            }

            switch ($name)
            {
                case 'edit':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', $name, $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'read', $name, $DATABASE['TABLE']);
                break;
                case 'delete':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', 'edit', $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'read', 'edit', $DATABASE['TABLE']);
                break;
                case 'new':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', 'read', $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'default', $name, $DATABASE['TABLE']);
                break;
            }

        }

        $AUTO_INTERFACE[$action] = $INTERFACE[$action];
        $AUTO_INTERFACE[$action]['TABLE']  = $DATABASE['TABLE'];
        $AUTO_INTERFACE[$action]['PRESET'] = strtolower($name);
        return $action;
    }
    
    function _createActionName($id, $type, $name, $table)
    {
        return $id."_".$type."_".$name."_".$table;
    }

}

//  2006, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>